@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2019 Intel Corporation
+// Copyright (c) 2017-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -33,6 +33,8 @@
 
 #include "umc_h265_va_supplier.h"
 #include "umc_va_linux_protected.h"
+#include "umc_va_video_processing.h"
+#include "mfx_session.h"
 
 using namespace UMC_HEVC_DECODER;
 
@@ -230,6 +232,58 @@ mfxStatus VideoDECODEH265::Init(mfxVideoParam *par)
 
     int32_t useInternal = (MFX_PLATFORM_SOFTWARE == m_platform) ?
         (m_vPar.IOPattern & MFX_IOPATTERN_OUT_VIDEO_MEMORY) : (m_vPar.IOPattern & MFX_IOPATTERN_OUT_SYSTEM_MEMORY);
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    mfxExtDecVideoProcessing * videoProcessing = (mfxExtDecVideoProcessing *)GetExtendedBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_DEC_VIDEO_PROCESSING);
+    /* There are following conditions for post processing via HW fixed function engine:
+     * (1): Progressive only for ICL and Before, interlace is supported for ICL following patforms
+     * (2): Supported on SKL (Core) and APL (Atom) platform and above
+     * (3): Only video memory supported (so, OPAQ memory does not supported!)
+     * */
+    if (videoProcessing)
+    {
+        MFX_CHECK(m_core->GetHWType() >= MFX_HW_SCL &&
+                  (m_vPar.IOPattern & MFX_IOPATTERN_OUT_VIDEO_MEMORY),
+                  MFX_ERR_UNSUPPORTED);
+        //PicStruct support differs, need to check per-platform
+        if (m_core->GetHWType() <= MFX_HW_ICL_LP)
+        {
+            MFX_CHECK(m_vPar.mfx.FrameInfo.PicStruct == MFX_PICSTRUCT_PROGRESSIVE, MFX_ERR_UNSUPPORTED);
+        }
+        MFX_CHECK(par->mfx.FrameInfo.FourCC == videoProcessing->Out.FourCC, MFX_ERR_UNSUPPORTED);//This is to avoid CSC cases, will remove once CSC is fully tested
+		bool is_fourcc_supported = false;
+        if (m_core->GetHWType() < MFX_HW_TGL_LP)
+        {
+            is_fourcc_supported =
+                      (  videoProcessing->Out.FourCC == MFX_FOURCC_RGB4
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_NV12
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_P010
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_YUY2
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_AYUV);
+        }
+        else
+        {
+            is_fourcc_supported =
+                      (  videoProcessing->Out.FourCC == MFX_FOURCC_RGB4
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_NV12
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_P010
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_YUY2
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_AYUV
+#if (MFX_VERSION >= 1027)
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_Y410
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_Y210
+#endif
+#if (MFX_VERSION >= 1031)
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_Y216
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_Y416
+                      || videoProcessing->Out.FourCC == MFX_FOURCC_P016
+#endif
+                      );
+        }
+       MFX_CHECK(is_fourcc_supported,MFX_ERR_UNSUPPORTED);
+        if (m_core->GetVAType() == MFX_HW_VAAPI)
+            useInternal = 1;
+    }
+#endif
 
     if (m_vPar.IOPattern & MFX_IOPATTERN_OUT_OPAQUE_MEMORY)
     {
@@ -325,6 +379,13 @@ mfxStatus VideoDECODEH265::Init(mfxVideoParam *par)
     UMC::Status umcSts = m_FrameAllocator->InitMfx(0, m_core, &m_vFirstPar, &request, &m_response, !useInternal, m_platform == MFX_PLATFORM_SOFTWARE);
     MFX_CHECK(umcSts == UMC::UMC_OK, MFX_ERR_MEMORY_ALLOC);
 
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    if (videoProcessing)
+    {
+        MFX_CHECK(useInternal || MFX_HW_D3D11 == m_core->GetVAType(), MFX_ERR_UNSUPPORTED);
+        m_FrameAllocator->SetSfcPostProcessingFlag(true);
+    }
+#endif
     umcSts = m_MemoryAllocator.InitMem(0, m_core);
     MFX_CHECK(umcSts == UMC::UMC_OK, MFX_ERR_MEMORY_ALLOC);
 
@@ -341,6 +402,13 @@ mfxStatus VideoDECODEH265::Init(mfxVideoParam *par)
         umcVideoParams.pVideoAccelerator = m_va;
         static_cast<VATaskSupplier*>(m_pH265VideoDecoder.get())->SetVideoHardwareAccelerator(m_va);
 
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+        if (m_va->GetVideoProcessingVA())
+        {
+            umcSts = m_va->GetVideoProcessingVA()->Init(par, videoProcessing);
+            MFX_CHECK(umcSts == UMC::UMC_OK, MFX_ERR_INVALID_VIDEO_PARAM);
+        }
+#endif
     }
 
     umcVideoParams.lpMemoryAllocator = &m_MemoryAllocator;
@@ -391,6 +459,20 @@ mfxStatus VideoDECODEH265::Reset(mfxVideoParam *par)
         type = m_core->GetHWType();
     }
 
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    mfxExtDecVideoProcessing * videoProcessing = (mfxExtDecVideoProcessing *)GetExtendedBuffer(m_vFirstPar.ExtParam, m_vFirstPar.NumExtParam, MFX_EXTBUFF_DEC_VIDEO_PROCESSING);
+    if (videoProcessing != nullptr)
+    {
+        // hardware resize is enabled
+        bool hardwareUpscale =
+            videoProcessing->Out.Width >= par->mfx.FrameInfo.Width ||
+            videoProcessing->Out.Height >= par->mfx.FrameInfo.Height;
+        if (hardwareUpscale)
+        {
+            return MFX_ERR_INVALID_VIDEO_PARAM;
+        }
+    }
+#endif
     eMFXPlatform platform = MFX_Utility::GetPlatform_H265(m_core, par);
 
     MFX_CHECK(CheckVideoParamDecoders(par, m_core->IsExternalFrameAllocator(), type) >= MFX_ERR_NONE, MFX_ERR_INVALID_VIDEO_PARAM);
@@ -509,6 +591,14 @@ mfxStatus VideoDECODEH265::GetVideoParam(mfxVideoParam *par)
         mfxExtVideoSignalInfo * videoSignalInternal = m_vPar.GetExtendedBuffer<mfxExtVideoSignalInfo>(MFX_EXTBUFF_VIDEO_SIGNAL_INFO);
         *videoSignal = *videoSignalInternal;
     }
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    mfxExtDecVideoProcessing * videoProcessing = (mfxExtDecVideoProcessing *)GetExtendedBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_DEC_VIDEO_PROCESSING);
+    if (videoProcessing)
+    {
+        mfxExtDecVideoProcessing * videoProcessingInternal = m_vPar.GetExtendedBuffer<mfxExtDecVideoProcessing>(MFX_EXTBUFF_DEC_VIDEO_PROCESSING);
+        *videoProcessing = *videoProcessingInternal;
+    }
+#endif
 
     mfxExtHEVCParam * hevcParam = (mfxExtHEVCParam *)GetExtendedBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_HEVC_PARAM);
     if (hevcParam)
@@ -974,6 +1064,20 @@ mfxStatus VideoDECODEH265::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1 *
         MFX_CHECK(m_va->GetProtectedVA() && (bs->DataFlag & MFX_BITSTREAM_COMPLETE_FRAME), MFX_ERR_UNDEFINED_BEHAVIOR);
         m_va->GetProtectedVA()->SetBitstream(bs);
     }
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    if (m_va->GetVideoProcessingVA() && m_core->GetVAType() == MFX_HW_VAAPI)
+    {
+        mfxHDL surfHDL = {};
+        if (!m_isOpaq)
+            sts = m_core->GetExternalFrameHDL(surface_work->Data.MemId, &surfHDL, false);
+        else
+            sts = m_core->GetFrameHDL(surface_work->Data.MemId, &surfHDL, false);
+        MFX_CHECK_STS(sts);
+        m_va->GetVideoProcessingVA()->SetOutputSurface(surfHDL);
+    }
+#endif // !MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    //gpu session priority
+    m_va->m_ContextPriority = m_core->GetSession()->m_priority;
 
     try
     {
@@ -1179,6 +1283,16 @@ void VideoDECODEH265::FillOutputSurface(mfxFrameSurface1 **surf_out, mfxFrameSur
     surface_out->Info.CropX = (mfxU16)(pFrame->m_crop_left);
     surface_out->Info.CropY = (mfxU16)(pFrame->m_crop_top);
 
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    mfxExtDecVideoProcessing * videoProcessing = (mfxExtDecVideoProcessing *)GetExtendedBuffer(m_vFirstPar.ExtParam, m_vFirstPar.NumExtParam, MFX_EXTBUFF_DEC_VIDEO_PROCESSING);
+    if (videoProcessing)
+    {
+        surface_out->Info.CropH = videoProcessing->Out.CropH;
+        surface_out->Info.CropW = videoProcessing->Out.CropW;
+        surface_out->Info.CropX = videoProcessing->Out.CropX;
+        surface_out->Info.CropY = videoProcessing->Out.CropY;
+    }
+#endif
     bool isShouldUpdate = !(m_vFirstPar.mfx.FrameInfo.AspectRatioH || m_vFirstPar.mfx.FrameInfo.AspectRatioW);
 
     surface_out->Info.AspectRatioH = isShouldUpdate ? (mfxU16)pFrame->m_aspect_height : m_vFirstPar.mfx.FrameInfo.AspectRatioH;
@@ -1534,6 +1648,34 @@ bool VideoDECODEH265::IsSameVideoParam(mfxVideoParam * newPar, mfxVideoParam * o
             return false;
     }
 
+#ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
+    mfxExtDecVideoProcessing * newVideoProcessing = (mfxExtDecVideoProcessing *)GetExtendedBuffer(newPar->ExtParam, newPar->NumExtParam, MFX_EXTBUFF_DEC_VIDEO_PROCESSING);
+    mfxExtDecVideoProcessing * oldVideoProcessing = (mfxExtDecVideoProcessing *)GetExtendedBuffer(oldPar->ExtParam, oldPar->NumExtParam, MFX_EXTBUFF_DEC_VIDEO_PROCESSING);
+    if (((newVideoProcessing) && (!oldVideoProcessing)) ||
+        ((!newVideoProcessing) && (oldVideoProcessing)))
+        return false;
+    else if (newVideoProcessing && oldVideoProcessing)
+    {
+        if (newVideoProcessing->Out.Width > oldVideoProcessing->Out.Width)
+            return false;
+        if (newVideoProcessing->Out.Height > oldVideoProcessing->Out.Height)
+            return false;
+        if (!((newVideoProcessing->In.CropX <= newVideoProcessing->In.CropW) &&
+            (newVideoProcessing->In.CropW <= newPar->mfx.FrameInfo.CropW) &&
+            (newVideoProcessing->In.CropY <= newVideoProcessing->In.CropH) &&
+            (newVideoProcessing->In.CropH <= newPar->mfx.FrameInfo.CropH)))
+            return false;
+        if (!((newVideoProcessing->Out.CropX <= newVideoProcessing->Out.CropW) &&
+            (newVideoProcessing->Out.CropW <= newVideoProcessing->Out.Width) &&
+            ((newVideoProcessing->Out.CropX + newVideoProcessing->Out.CropH)
+                <= newVideoProcessing->Out.Width) &&
+                (newVideoProcessing->Out.CropY <= newVideoProcessing->Out.CropH) &&
+            (newVideoProcessing->Out.CropH <= newVideoProcessing->Out.Height) &&
+            ((newVideoProcessing->Out.CropY + newVideoProcessing->Out.CropH)
+                <= newVideoProcessing->Out.Height)))
+            return false;
+    }
+#endif //MFX_DEC_VIDEO_POSTPROCESS_DISABLE
     return true;
 }
 
