@@ -1,4 +1,4 @@
-// Copyright (c) 2019 Intel Corporation
+// Copyright (c) 2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -657,6 +657,11 @@ mfxStatus VideoDECODEMPEG2::DecodeFrameCheck(mfxBitstream* bs, mfxFrameSurface1*
     mfxStatus sts = bs ? CheckBitstream(bs) : MFX_ERR_NONE;
     MFX_CHECK_STS(sts);
 
+    //gpu session priority
+    UMC_MPEG2_DECODER::MPEG2DecoderParams* mpeg2DecoderParams = m_decoder->GetMpeg2DecoderParams();
+    if (mpeg2DecoderParams != nullptr && mpeg2DecoderParams->pVideoAccelerator != nullptr)
+        mpeg2DecoderParams->pVideoAccelerator->m_ContextPriority = m_core->GetSession()->m_priority;
+
     // in case of EOS (flushing) decoder may return buffered surface
     // without surface_work
     if (surface_work != nullptr)
@@ -751,6 +756,8 @@ mfxStatus VideoDECODEMPEG2::GetPayload( mfxU64 *ts, mfxPayload *payload)
 
     const auto msg = payloads->GetPayloadMessage();
 
+    static const mfxU16 USER_DATA_START_CODE = 0x000001B2;
+
     if (msg)
     {
         if (payload->BufSize < msg->msg_size)
@@ -761,7 +768,7 @@ mfxStatus VideoDECODEMPEG2::GetPayload( mfxU64 *ts, mfxPayload *payload)
         std::copy(msg->data, msg->data + msg->msg_size, payload->Data);
 
         payload->NumBit = (mfxU32)(msg->msg_size * 8);
-        payload->Type = UMC_MPEG2_DECODER::USER_DATA;
+        payload->Type = USER_DATA_START_CODE;
     }
     else
     {
