@@ -67,6 +67,12 @@ static const GUID DXVA2_Intel_LowpowerEncode_HEVC_Main444 =
 static const GUID DXVA2_Intel_LowpowerEncode_HEVC_Main444_10 =
 { 0x10e19ac8, 0xbf39, 0x4443, { 0xbe, 0xc3, 0x1b, 0x0c, 0xbf, 0xe4, 0xc7, 0xaa } };
 
+static const GUID DXVA2_Intel_Encode_HEVC_Main12 =
+{ 0xd6d6bc4f, 0xd51a, 0x4712, { 0x97, 0xe8, 0x75, 0x9, 0x17, 0xc8, 0x60, 0xfd } };
+static const GUID DXVA2_Intel_Encode_HEVC_Main422_12 =
+{ 0x7fef652d, 0x3233, 0x44df, { 0xac, 0xf7, 0xec, 0xfb, 0x58, 0x4d, 0xab, 0x35 } };
+static const GUID DXVA2_Intel_Encode_HEVC_Main444_12 =
+{ 0xf8fa34b7, 0x93f5, 0x45a4, { 0xbf, 0xc0, 0x38, 0x17, 0xce, 0xe6, 0xbb, 0x93 } };
 GUID GetGUID(MfxVideoParam const & par);
 
 const GUID GuidTable[3][3][3] =
@@ -87,6 +93,11 @@ const GUID GuidTable[3][3][3] =
         },
         // BitDepthLuma = 12
         {
+#if (MFX_VERSION >= 1031)
+            /*420*/ DXVA2_Intel_Encode_HEVC_Main12,
+            /*422*/ DXVA2_Intel_Encode_HEVC_Main422_12,
+            /*444*/ DXVA2_Intel_Encode_HEVC_Main444_12
+#endif
         }
     },
     // LowPower = ON
@@ -108,7 +119,7 @@ const GUID GuidTable[3][3][3] =
         }
     }
 };
-mfxStatus HardcodeCaps(MFX_ENCODE_CAPS_HEVC& caps, VideoCORE* pCore);
+mfxStatus HardcodeCaps(MFX_ENCODE_CAPS_HEVC& caps, VideoCORE* pCore, MfxVideoParam const &par);
 
 class DriverEncoder;
 
@@ -119,7 +130,7 @@ typedef enum tagENCODER_TYPE
 } ENCODER_TYPE;
 
 DriverEncoder* CreatePlatformH265Encoder(VideoCORE* core, ENCODER_TYPE type = ENCODER_DEFAULT);
-mfxStatus QueryHwCaps(VideoCORE* core, GUID guid, MFX_ENCODE_CAPS_HEVC & caps, mfxU32 width, mfxU32 height);
+mfxStatus QueryHwCaps(VideoCORE* core, GUID guid, MFX_ENCODE_CAPS_HEVC & caps, MfxVideoParam const & par);
 mfxStatus CheckHeaders(MfxVideoParam const & par, MFX_ENCODE_CAPS_HEVC const & caps);
 
 mfxStatus FillCUQPDataDDI(Task& task, MfxVideoParam &par, VideoCORE& core, mfxFrameInfo &CUQPFrameInfo);
@@ -140,8 +151,7 @@ public:
     mfxStatus CreateAuxilliaryDevice(
                     VideoCORE * core,
                     GUID        guid,
-                    mfxU32      width,
-                    mfxU32      height) = 0;
+                    MfxVideoParam const & par) = 0;
 
     virtual
     mfxStatus CreateAccelerationService(
