@@ -1,15 +1,15 @@
-// Copyright (c) 2018-2019 Intel Corporation
-// 
+// Copyright (c) 2018-2020 Intel Corporation
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in all
 // copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -21,6 +21,10 @@
 #ifndef __MFX_H264_ENC_COMMON_HW_H__
 #define __MFX_H264_ENC_COMMON_HW_H__
 
+#if defined(AS_H264LA_PLUGIN) && defined(MFX_ENABLE_ENCTOOLS)
+#undef MFX_ENABLE_ENCTOOLS
+#endif
+
 #include "mfx_common.h"
 #include "mfxla.h"
 
@@ -31,8 +35,11 @@
 #include <memory>
 #include "mfx_ext_buffers.h"
 #include "mfxfei.h"
+#ifdef MFX_ENABLE_ENCTOOLS
+#include "mfxenctools-int.h"
+#else
 #include "mfxbrc.h"
-
+#endif
 #include "mfx_h264_encode_struct_vaapi.h"
 
 #if defined(MFX_VA_LINUX)
@@ -43,6 +50,8 @@
 
 #include "umc_defs.h"
 
+#define ENABLE_APQ_LQ
+
 
 #define D3DFMT_NV12 (D3DFORMAT)(MFX_MAKEFOURCC('N', 'V', '1', '2'))
 #define D3DDDIFMT_NV12 (D3DDDIFORMAT)(MFX_MAKEFOURCC('N', 'V', '1', '2'))
@@ -215,6 +224,23 @@ namespace MfxHwH264Encode
     BIND_EXTBUF_TYPE_TO_ID (mfxExtMultiFrameControl,     MFX_EXTBUFF_MULTI_FRAME_CONTROL     );
     BIND_EXTBUF_TYPE_TO_ID (mfxExtMultiFrameParam,       MFX_EXTBUFF_MULTI_FRAME_PARAM       );
 #endif
+#if defined (MFX_ENABLE_ENCTOOLS)
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncTools, MFX_EXTBUFF_ENCTOOLS);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsBRCFrameParams, MFX_EXTBUFF_ENCTOOLS_BRC_FRAME_PARAM);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsBRCQuantControl, MFX_EXTBUFF_ENCTOOLS_BRC_QUANT_CONTROL);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsBRCHRDPos, MFX_EXTBUFF_ENCTOOLS_BRC_HRD_POS);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsBRCEncodeResult, MFX_EXTBUFF_ENCTOOLS_BRC_ENCODE_RESULT);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsBRCStatus, MFX_EXTBUFF_ENCTOOLS_BRC_STATUS);
+    BIND_EXTBUF_TYPE_TO_ID(mfxExtEncToolsConfig, MFX_EXTBUFF_ENCTOOLS_CONFIG);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsCtrlExtDevice, MFX_EXTBUFF_ENCTOOLS_DEVICE);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsCtrlExtAllocator, MFX_EXTBUFF_ENCTOOLS_ALLOCATOR);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsFrameToAnalyze, MFX_EXTBUFF_ENCTOOLS_FRAME_TO_ANALYZE);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsHintPreEncodeSceneChange, MFX_EXTBUFF_ENCTOOLS_HINT_SCENE_CHANGE);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsHintPreEncodeGOP, MFX_EXTBUFF_ENCTOOLS_HINT_GOP);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsHintPreEncodeARefFrames, MFX_EXTBUFF_ENCTOOLS_HINT_AREF);
+    BIND_EXTBUF_TYPE_TO_ID(mfxEncToolsBRCBufferHint, MFX_EXTBUFF_ENCTOOLS_BRC_BUFFER_HINT);
+
+#endif
 
 #undef BIND_EXTBUF_TYPE_TO_ID
 
@@ -253,7 +279,26 @@ namespace MfxHwH264Encode
         for (mfxU32 i = 0; i < 16; i++)
             extBuf.LongTermRefList[i].FrameOrder = mfxU32(MFX_FRAMEORDER_UNKNOWN);
     }
+#if defined(MFX_ENABLE_ENCTOOLS)
+    template <> inline void InitExtBufHeader<mfxExtEncToolsConfig>(mfxExtEncToolsConfig & extBuf)
+    {
+        Zero(extBuf);
+        extBuf.Header.BufferId = ExtBufTypeToId<mfxExtEncToolsConfig>::id;
+        extBuf.Header.BufferSz = sizeof(mfxExtEncToolsConfig);
 
+        extBuf.AdaptiveI =
+            extBuf.AdaptiveB =
+            extBuf.AdaptiveRefP =
+            extBuf.AdaptiveRefB =
+            extBuf.SceneChange =
+            extBuf.AdaptiveLTR =
+            extBuf.AdaptivePyramidQuantP =
+            extBuf.AdaptivePyramidQuantB =
+            extBuf.AdaptiveQuantMatrices =
+            extBuf.BRCBufferHints =
+            extBuf.BRC = MFX_CODINGOPTION_OFF;
+    }
+#endif
     template <class T> struct GetPointedType {};
     template <class T> struct GetPointedType<T *> { typedef T Type; };
     template <class T> struct GetPointedType<T const *> { typedef T Type; };
@@ -519,13 +564,18 @@ namespace MfxHwH264Encode
 
         void ApplyDefaultsToMvcSeqDesc();
 
+
     protected:
         void Construct(mfxVideoParam const & par);
 
         void ConstructMvcSeqDesc(mfxExtMVCSeqDesc const & desc);
 
     private:
+#if defined(MFX_ENABLE_ENCTOOLS)
+        mfxExtBuffer *              m_extParam[40];
+#else
         mfxExtBuffer *              m_extParam[32];
+#endif
         // external, documented
         mfxExtCodingOption          m_extOpt;
         mfxExtCodingOption2         m_extOpt2;
@@ -556,7 +606,12 @@ namespace MfxHwH264Encode
 #if defined(__MFXBRC_H__)
         mfxExtBRC                   m_extBRC;
 #endif
-
+#if defined(MFX_ENABLE_ENCTOOLS)
+        mfxEncTools                     m_encTools;
+        mfxExtEncToolsConfig            m_encToolsConfig;
+        mfxEncToolsCtrlExtDevice        m_extDevice;
+        mfxEncToolsCtrlExtAllocator     m_extAllocator;
+#endif
 #if defined (MFX_ENABLE_MFE)
         mfxExtMultiFrameParam    m_MfeParam;
         mfxExtMultiFrameControl  m_MfeControl;
@@ -606,9 +661,12 @@ namespace MfxHwH264Encode
             mfxU16 widthLa;
             mfxU16 heightLa;
             mfxU32 TCBRCTargetFrameSize;
+            mfxU32 PPyrInterval;
         } calcParam;
     };
 
+    bool   isSWBRC (MfxVideoParam const & par);
+    bool   isAdaptiveQP(MfxVideoParam const & par);
     mfxU16 GetMaxNumSlices(MfxVideoParam const & par);
 
     mfxU16 GetNumSurfInput(MfxVideoParam const & video);
@@ -639,8 +697,15 @@ namespace MfxHwH264Encode
     bool IsLookAheadSupported(
         MfxVideoParam const & video,
         eMFXHWType            platform);
+    bool IsMctfSupported(
+        MfxVideoParam const & video);
 
-    bool  IsExtBrcSceneChangeSupported(
+    mfxU32 GetPPyrSize(
+        MfxVideoParam const & video,
+        mfxU32 miniGopSize,
+        bool   bEncToolsLA);
+
+    bool IsExtBrcSceneChangeSupported(
         MfxVideoParam const & video);
 
     bool IsCmNeededForSCD(
@@ -805,7 +870,6 @@ namespace MfxHwH264Encode
         mfxU32          numExtBuf,
         mfxU32          id,
         mfxU32          offset = 0);
-
     struct mfxExtBufferProxy;
     struct mfxExtBufferRefProxy;
     template <typename T> mfxExtBufferProxy GetExtBuffer(const T & par, mfxU32 fieldId = 0);
@@ -942,6 +1006,15 @@ namespace MfxHwH264Encode
 
     ENCODE_FRAME_SIZE_TOLERANCE ConvertLowDelayBRCMfx2Ddi(mfxU16 type, bool bTCBRC);
 
+    enum class SliceDividerType
+    {
+        ONESLICE            = 0, // Once slice for the whole frame
+        ROW2ROW             = 1, // Slices are power of 2 number of rows, all slices the same
+        ROWSLICE            = 2, // Slices are any number of rows, all slices the same
+        ARBITRARY_ROW_SLICE = 3, // Slices are any number of rows, slices can be different
+        ARBITRARY_MB_SLICE  = 4, // Slices are any number of MBs, slices can be different
+    };
+
     struct MfxMemId
     {
     public:
@@ -1169,9 +1242,9 @@ namespace MfxHwH264Encode
     };
 
 
-    struct SliceDividerBluRay : SliceDivider
+    struct SliceDividerArbitraryRowSlice : SliceDivider
     {
-        SliceDividerBluRay(
+        SliceDividerArbitraryRowSlice(
             mfxU32 numSlice,
             mfxU32 widthInMbs,
             mfxU32 heightInMbs);
@@ -1180,9 +1253,9 @@ namespace MfxHwH264Encode
     };
 
 
-    struct SliceDividerSnb : SliceDivider
+    struct SliceDividerRow2Row : SliceDivider
     {
-        SliceDividerSnb(
+        SliceDividerRow2Row(
             mfxU32 numSlice,
             mfxU32 widthInMbs,
             mfxU32 heightInMbs);
@@ -1193,9 +1266,9 @@ namespace MfxHwH264Encode
     };
 
 
-    struct SliceDividerHsw : SliceDivider
+    struct SliceDividerRowSlice : SliceDivider
     {
-        SliceDividerHsw(
+        SliceDividerRowSlice(
             mfxU32 numSlice,
             mfxU32 widthInMbs,
             mfxU32 heightInMbs);
@@ -1231,7 +1304,7 @@ namespace MfxHwH264Encode
         static bool Next(SliceDividerState & state);
     };
     SliceDivider MakeSliceDivider(
-        mfxU32  sliceHwCaps,
+        SliceDividerType sliceHwCaps,
         mfxU32  sliceSizeInMbs,
         mfxU32  numSlice,
         mfxU32  widthInMbs,
