@@ -916,8 +916,43 @@ mfxStatus VideoVPPBase::Query(VideoCORE * core, mfxVideoParam *in, mfxVideoParam
 
         } // if (in->ExtParam && out->ExtParam && (in->NumExtParam == out->NumExtParam) )
 
+        if (core->GetHWType() < MFX_HW_ICL)
+        {
+            if (out->vpp.Out.FourCC == MFX_FOURCC_P010 &&
+                out->vpp.In.FourCC  != MFX_FOURCC_NV12 &&
+                out->vpp.In.FourCC  != MFX_FOURCC_YV12 &&
+                out->vpp.In.FourCC  != MFX_FOURCC_YUY2 &&
+                out->vpp.In.FourCC  != MFX_FOURCC_RGB4 &&
+                out->vpp.In.FourCC  != MFX_FOURCC_P010) {
+                if( out->vpp.In.FourCC )
+                {
+                    out->vpp.In.FourCC = 0;
+                    mfxSts = MFX_ERR_UNSUPPORTED;
+                }
+            }
+        }
+
+#if (MFX_VERSION >= 1031)
+        if (core->GetHWType() <= MFX_HW_ICL_LP)
+        {
+            if (out->vpp.In.FourCC == MFX_FOURCC_P016 ||
+                out->vpp.In.FourCC == MFX_FOURCC_Y216 ||
+                out->vpp.In.FourCC == MFX_FOURCC_Y416 ||
+                out->vpp.Out.FourCC == MFX_FOURCC_P016 ||
+                out->vpp.Out.FourCC == MFX_FOURCC_Y216 ||
+                out->vpp.Out.FourCC == MFX_FOURCC_Y416) {
+                if( out->vpp.In.FourCC )
+                {
+                    out->vpp.In.FourCC = 0;
+                    mfxSts = MFX_ERR_UNSUPPORTED;
+                }
+            }
+        }
+#endif
+
         if ( out->vpp.In.FourCC  != MFX_FOURCC_P010 &&
              out->vpp.In.FourCC  != MFX_FOURCC_P210 &&
+             out->vpp.In.FourCC  != MFX_FOURCC_A2RGB10 &&
              out->vpp.Out.FourCC == MFX_FOURCC_A2RGB10 ){
             if( out->vpp.In.FourCC )
             {
@@ -969,7 +1004,9 @@ mfxStatus VideoVPPBase::Query(VideoCORE * core, mfxVideoParam *in, mfxVideoParam
             out->vpp.In.FourCC != MFX_FOURCC_Y216 &&
             out->vpp.In.FourCC != MFX_FOURCC_Y416 &&
 #endif
-            out->vpp.In.FourCC != MFX_FOURCC_AYUV)
+            out->vpp.In.FourCC != MFX_FOURCC_AYUV &&
+            // A2RGB10 supported as input in case of passthru copy
+            out->vpp.In.FourCC != MFX_FOURCC_A2RGB10 )
         {
             if( out->vpp.In.FourCC )
             {
