@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2019 Intel Corporation
+// Copyright (c) 2018-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -344,7 +344,7 @@ bool IsCompositionMode(mfxVideoParam* pParam)
 
 mfxStatus ExtendedQuery(VideoCORE * core, mfxU32 filterName, mfxExtBuffer* pHint)
 {
-    mfxStatus sts;
+    mfxStatus sts = MFX_ERR_NONE;
     /* Lets find out VA type (Linux, Win or Android) and platform type */
     /* It can be different behaviour for Linux and IVB, Linux and HSW*/
     bool bLinuxAndIVB_HSW_BDW = false;
@@ -404,6 +404,10 @@ mfxStatus ExtendedQuery(VideoCORE * core, mfxU32 filterName, mfxExtBuffer* pHint
             sts = MFX_ERR_NONE;
         }
     }
+    else if (MFX_EXTBUFF_VPP_SCALING == filterName)
+    {
+        sts = CheckScalingParam(pHint);
+    }
     else // ignore
     {
         sts = MFX_ERR_NONE;
