@@ -1,5 +1,5 @@
 /*//////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2018 Intel Corporation
+// Copyright (c) 2018-2020 Intel Corporation
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -19,8 +19,146 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 */
+
+#if defined(__SSE4_1__) || defined(_WIN32)
+
 #include "asc_sse4_impl.h"
 
+#define _mm_loadh_epi64(a, ptr) _mm_castps_si128(_mm_loadh_pi(_mm_castsi128_ps(a), (__m64 *)(ptr)))
+#define _mm_movehl_epi64(a, b) _mm_castps_si128(_mm_movehl_ps(_mm_castsi128_ps(a), _mm_castsi128_ps(b)))
+
+// Load 0..3 floats to XMM register from memory
+// NOTE: elements of XMM are permuted [ 2 - 1 ]
+__m128 LoadPartialXmm(
+    float  * pSrc,
+    mfxI32   len
+)
+{
+    __m128 xmm = _mm_setzero_ps();
+    if (len & 2)
+    {
+        xmm = _mm_loadh_pi(xmm, (__m64 *)pSrc);
+        pSrc += 2;
+    }
+    if (len & 1)
+    {
+        xmm = _mm_move_ss(xmm, _mm_load_ss(pSrc));
+    }
+    return xmm;
+}
+
+// Store 0..3 floats from XMM register to memory
+// NOTE: elements of XMM are permuted [ 2 - 1 ]
+void StorePartialXmm(
+    float  * pDst,
+    __m128   xmm,
+    mfxI32   len
+)
+{
+    if (len & 2)
+    {
+        _mm_storeh_pi((__m64 *)pDst, xmm);
+        pDst += 2;
+    }
+    if (len & 1)
+    {
+        _mm_store_ss(pDst, xmm);
+    }
+}
+
+// Load 0..15 bytes to XMM register from memory
+// NOTE: elements of XMM are permuted [ 8 4 2 - 1 ]
+template <char init>
+__m128i LoadPartialXmm(
+    unsigned char * pSrc,
+    mfxI32          len
+)
+{
+    __m128i xmm = _mm_set1_epi8(init);
+    if (len & 8) {
+        xmm = _mm_loadh_epi64(xmm, (__m64 *)pSrc);
+        pSrc += 8;
+    }
+    if (len & 4) {
+        xmm = _mm_insert_epi32(xmm, *((int *)pSrc), 1);
+        pSrc += 4;
+    }
+    if (len & 2) {
+        xmm = _mm_insert_epi16(xmm, *((short *)pSrc), 1);
+        pSrc += 2;
+    }
+    if (len & 1) {
+        xmm = _mm_insert_epi8(xmm, *pSrc, 0);
+    }
+    return xmm;
+}
+
+mfxU16 ME_SAD_8x8_Block_SSE4(mfxU8 *pSrc, mfxU8 *pRef, mfxU32 srcPitch, mfxU32 refPitch)
+{
+    __m128i s0 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pSrc[0 * srcPitch]), (__m128i *)&pSrc[1 * srcPitch]);
+    __m128i r0 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pRef[0 * refPitch]), (__m128i *)&pRef[1 * refPitch]);
+    __m128i s1 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pSrc[2 * srcPitch]), (__m128i *)&pSrc[3 * srcPitch]);
+    __m128i r1 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pRef[2 * refPitch]), (__m128i *)&pRef[3 * refPitch]);
+    __m128i s2 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pSrc[4 * srcPitch]), (__m128i *)&pSrc[5 * srcPitch]);
+    __m128i r2 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pRef[4 * refPitch]), (__m128i *)&pRef[5 * refPitch]);
+    __m128i s3 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pSrc[6 * srcPitch]), (__m128i *)&pSrc[7 * srcPitch]);
+    __m128i r3 = _mm_loadh_epi64(_mm_loadl_epi64((__m128i *)&pRef[6 * refPitch]), (__m128i *)&pRef[7 * refPitch]);
+
+    s0 = _mm_sad_epu8(s0, r0);
+    s1 = _mm_sad_epu8(s1, r1);
+    s2 = _mm_sad_epu8(s2, r2);
+    s3 = _mm_sad_epu8(s3, r3);
+
+    s0 = _mm_add_epi32(s0, s1);
+    s2 = _mm_add_epi32(s2, s3);
+
+    s0 = _mm_add_epi32(s0, s2);
+    s2 = _mm_movehl_epi64(s2, s0);
+    s0 = _mm_add_epi32(s0, s2);
+
+    return (mfxU16)_mm_extract_epi16(s0, 0);
+}
+
+void ME_VAR_8x8_Block_SSE4(mfxU8 *pSrc, mfxU8 *pRef, mfxU8 *pMCref, mfxI16 srcAvgVal, mfxI16 refAvgVal, mfxU32 srcPitch, mfxU32 refPitch, mfxI32 &var, mfxI32 &jtvar, mfxI32 &jtMCvar)
+{
+    __m128i srcAvg = _mm_set1_epi16(srcAvgVal);
+    __m128i refAvg = _mm_set1_epi16(refAvgVal);
+
+    __m128i src[8],
+        ref[8],
+        rmc[8],
+        accuVar = { 0 },
+        accuJtvar = { 0 },
+        accuMcJtvar = { 0 };
+
+    for (mfxU8 i = 0; i < 8; i++)
+    {
+        src[i] = _mm_cvtepu8_epi16(_mm_loadu_si128((__m128i *)&pSrc[i * srcPitch]));
+        ref[i] = _mm_cvtepu8_epi16(_mm_loadu_si128((__m128i *)&pRef[i * refPitch]));
+        rmc[i] = _mm_cvtepu8_epi16(_mm_loadu_si128((__m128i *)&pMCref[i * refPitch]));
+        src[i] = _mm_sub_epi16(src[i], srcAvg);
+        ref[i] = _mm_sub_epi16(ref[i], refAvg);
+        rmc[i] = _mm_sub_epi16(rmc[i], refAvg);
+        accuVar = _mm_add_epi32(_mm_madd_epi16(src[i], src[i]), accuVar);
+        accuJtvar = _mm_add_epi32(_mm_madd_epi16(src[i], ref[i]), accuJtvar);
+        accuMcJtvar = _mm_add_epi32(_mm_madd_epi16(src[i], rmc[i]), accuMcJtvar);
+    }
+
+    accuVar = _mm_hadd_epi32(accuVar, accuVar);
+    accuVar = _mm_hadd_epi32(accuVar, accuVar);
+
+    accuJtvar = _mm_hadd_epi32(accuJtvar, accuJtvar);
+    accuJtvar = _mm_hadd_epi32(accuJtvar, accuJtvar);
+
+    accuMcJtvar = _mm_hadd_epi32(accuMcJtvar, accuMcJtvar);
+    accuMcJtvar = _mm_hadd_epi32(accuMcJtvar, accuMcJtvar);
+
+    var += _mm_extract_epi32(accuVar, 0);
+    jtvar += _mm_extract_epi32(accuJtvar, 0);
+    jtMCvar += _mm_extract_epi32(accuMcJtvar, 0);
+}
+
+
 void ME_SAD_8x8_Block_Search_SSE4(mfxU8 *pSrc, mfxU8 *pRef, int pitch, int xrange, int yrange,
     mfxU16 *bestSAD, int *bestX, int *bestY)
 {
@@ -477,3 +615,4 @@ mfxI16 AvgLumaCalc_SSE4(pmfxU32 pAvgLineVal, int len) {
     return avgVal;
 }
 
+#endif // __SSE4_1__ || _WIN32
