@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2018 Intel Corporation
+// Copyright (c) 2017-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -135,6 +135,7 @@ namespace MfxHwVideoProcessing
         VABufferID m_deintFilterID;
         VABufferID m_procampFilterID;
         VABufferID m_frcFilterID;
+        VABufferID m_gpuPriorityID;
         mfxU32     m_deintFrameCount;
         VASurfaceID m_refForFRC[5];
 
@@ -146,6 +147,8 @@ namespace MfxHwVideoProcessing
 
         std::set<mfxU32> m_cachedReadyTaskIndex;
 
+        mfxU32 m_MaxContextPriority;
+
         typedef struct
         {
             VASurfaceID surface;
