@@ -387,7 +387,7 @@ Status VATaskSupplier::AllocateFrameData(H264DecoderFrame * pFrame)
 H264Slice * VATaskSupplier::DecodeSliceHeader(NalUnit *nalUnit)
 {
     size_t dataSize = nalUnit->GetDataSize();
-    nalUnit->SetDataSize(MFX_MIN(1024, dataSize));
+    nalUnit->SetDataSize(std::min<size_t>(1024, dataSize));
 
     H264Slice * slice = TaskSupplier::DecodeSliceHeader(nalUnit);
 
