@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2018 Intel Corporation
+// Copyright (c) 2017-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -339,6 +339,36 @@ Status MJPEGVideoDecoderMFX_HW::DefaultInitializationHuffmantables()
     return UMC_OK;
 }
 
+Status MJPEGVideoDecoderMFX_HW::PackPriorityParams()
+{
+    Status sts = UMC_OK;
+    mfxPriority priority = m_va->m_ContextPriority;
+    UMCVACompBuffer *GpuPriorityBuf;
+    VAContextParameterUpdateBuffer* GpuPriorityBuf_JpegDecode = (VAContextParameterUpdateBuffer *)m_va->GetCompBuffer(VAContextParameterUpdateBufferType, &GpuPriorityBuf, sizeof(VAContextParameterUpdateBuffer));
+    if (!GpuPriorityBuf_JpegDecode)
+        return UMC_ERR_DEVICE_FAILED;
+
+    memset(GpuPriorityBuf_JpegDecode, 0, sizeof(VAContextParameterUpdateBuffer));
+    GpuPriorityBuf_JpegDecode->flags.bits.context_priority_update = 1;
+    if(priority == MFX_PRIORITY_LOW)
+    {
+        GpuPriorityBuf_JpegDecode->context_priority.bits.priority = 0;
+    }
+    else if (priority == MFX_PRIORITY_HIGH)
+    {
+        GpuPriorityBuf_JpegDecode->context_priority.bits.priority = m_va->m_MaxContextPriority;
+    }
+    else
+    {
+        GpuPriorityBuf_JpegDecode->context_priority.bits.priority = m_va->m_MaxContextPriority/2;
+    }
+
+    GpuPriorityBuf->SetDataSize(sizeof(VAContextParameterUpdateBuffer));
+
+    return sts;
+
+}
+
 Status MJPEGVideoDecoderMFX_HW::GetFrameHW(MediaDataEx* in)
 {
 #ifdef UMC_VA
@@ -511,6 +541,13 @@ Status MJPEGVideoDecoderMFX_HW::GetFrameHW(MediaDataEx* in)
         }
     }
 
+    if(m_va->m_MaxContextPriority)
+    {
+        sts = PackPriorityParams();
+        if (sts != UMC_OK)
+            return sts;
+    }
+
     /////////////////////////////////////////////////////////////////////////////////////////
     sts = m_va->Execute();
     if (sts != UMC_OK)
