@@ -1,4 +1,4 @@
-// Copyright (c) 2012-2019 Intel Corporation
+// Copyright (c) 2012-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -28,6 +28,8 @@
 #include "mfx_win_reg_key.h"
 #endif
 
+#include "mfx_driver_store_loader.h"
+
 #include "mfx_dispatcher.h"
 
 namespace MFX
@@ -36,13 +38,26 @@ namespace MFX
 // declare desired storage ID
 enum
 {
+#if defined (MFX_TRACER_WA_FOR_DS)
     MFX_UNKNOWN_KEY             = -1,
-    MFX_CURRENT_USER_KEY        = 0,
-    MFX_LOCAL_MACHINE_KEY       = 1,
-    MFX_APP_FOLDER              = 2,
-    MFX_PATH_MSDK_FOLDER        = 3,
-    MFX_STORAGE_ID_FIRST    = MFX_CURRENT_USER_KEY,
+    MFX_TRACER                  = 0,
+    MFX_DRIVER_STORE            = 1,
+    MFX_CURRENT_USER_KEY        = 2,
+    MFX_LOCAL_MACHINE_KEY       = 3,
+    MFX_APP_FOLDER              = 4,
+    MFX_PATH_MSDK_FOLDER        = 5,
+    MFX_STORAGE_ID_FIRST = MFX_TRACER,
+    MFX_STORAGE_ID_LAST = MFX_PATH_MSDK_FOLDER
+#else
+    MFX_UNKNOWN_KEY             = -1,
+    MFX_DRIVER_STORE            = 0,
+    MFX_CURRENT_USER_KEY        = 1,
+    MFX_LOCAL_MACHINE_KEY       = 2,
+    MFX_APP_FOLDER              = 3,
+    MFX_PATH_MSDK_FOLDER        = 4,
+    MFX_STORAGE_ID_FIRST    = MFX_DRIVER_STORE,
     MFX_STORAGE_ID_LAST     = MFX_PATH_MSDK_FOLDER
+#endif
 };
 
 // Try to initialize using given implementation type. Select appropriate type automatically in case of MFX_IMPL_VIA_ANY.
@@ -79,9 +94,13 @@ protected:
     void Release(void);
 
     // Initialize the registry iterator
-    mfxStatus InitRegistry(eMfxImplType implType, mfxIMPL implInterface, const mfxU32 adapterNum, int storageID);
+    mfxStatus InitRegistry(int storageID);
+#if defined(MFX_TRACER_WA_FOR_DS)
+    // Initialize the registry iterator for searching for tracer
+    mfxStatus InitRegistryTracer();
+#endif
     // Initialize the app/module folder iterator
-    mfxStatus InitFolder(eMfxImplType implType, mfxIMPL implInterface, const mfxU32 adapterNum, const wchar_t * path, const int storageID);
+    mfxStatus InitFolder(eMfxImplType implType, const wchar_t * path, const int storageID);
 
 
     eMfxImplType m_implType;                                    // Required library implementation
@@ -102,6 +121,8 @@ protected:
 
     wchar_t  m_path[msdk_disp_path_len];
 
+    DriverStoreLoader m_driverStoreLoader;                      // for loading MediaSDK from DriverStore
+
 private:
     // unimplemented by intent to make this class non-copyable
     MFXLibraryIterator(const MFXLibraryIterator &);
