@@ -1,4 +1,4 @@
-// Copyright (c) 2012-2019 Intel Corporation
+// Copyright (c) 2012-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -18,7 +18,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include <Windows.h>
+#include <windows.h>
 #include <stringapiset.h>
 
 #include <new>
@@ -31,7 +31,7 @@
 #include "mfx_critical_section.h"
 
 #if defined(MEDIASDK_UWP_DISPATCHER)
-#include "mfx_driver_store_loader.h"
+#include "mfx_dispatcher_uwp.h"
 #endif
 
 #include <string.h> /* for memset on Linux */
@@ -69,10 +69,10 @@ namespace
         {MFX_LIB_SOFTWARE, MFX_IMPL_SOFTWARE | MFX_IMPL_AUDIO,  0},
 #if (MFX_VERSION >= MFX_VERSION_NEXT)
         //MFX_SINGLE_THREAD case
-        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE|MFX_IMPL_EXTERNAL_THREADING, 0},
-        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE2|MFX_IMPL_EXTERNAL_THREADING, 1},
-        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE3|MFX_IMPL_EXTERNAL_THREADING, 2},
-        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE4|MFX_IMPL_EXTERNAL_THREADING, 3},
+        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE | MFX_IMPL_EXTERNAL_THREADING, 0},
+        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE2 | MFX_IMPL_EXTERNAL_THREADING, 1},
+        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE3 | MFX_IMPL_EXTERNAL_THREADING, 2},
+        {MFX_LIB_HARDWARE, MFX_IMPL_HARDWARE4 | MFX_IMPL_EXTERNAL_THREADING, 3},
 #endif
     };
 
@@ -115,7 +115,7 @@ using namespace MFX;
 // All other functions are implemented implicitly.
 //
 
-typedef MFXVector<MFX_DISP_HANDLE*> HandleVector;
+typedef MFXVector<MFX_DISP_HANDLE_EX*> HandleVector;
 typedef MFXVector<mfxStatus>        StatusVector;
 
 struct VectorHandleGuard
@@ -137,11 +137,32 @@ private:
 };
 
 
-int HandleSort (const void * plhs, const void * prhs)
+static int HandleSort (const void * plhs, const void * prhs)
 {
-    const MFX_DISP_HANDLE * lhs = *(const MFX_DISP_HANDLE **)plhs;
-    const MFX_DISP_HANDLE * rhs = *(const MFX_DISP_HANDLE **)prhs;
+    const MFX_DISP_HANDLE_EX * lhs = *(const MFX_DISP_HANDLE_EX **)plhs;
+    const MFX_DISP_HANDLE_EX * rhs = *(const MFX_DISP_HANDLE_EX **)prhs;
 
+    // prefer HW implementation
+    if (lhs->implType != MFX_LIB_HARDWARE && rhs->implType == MFX_LIB_HARDWARE)
+    {
+        return 1;
+    }
+    if (lhs->implType == MFX_LIB_HARDWARE && rhs->implType != MFX_LIB_HARDWARE)
+    {
+        return -1;
+    }
+
+    // prefer integrated GPU
+    if (lhs->mediaAdapterType != MFX_MEDIA_INTEGRATED && rhs->mediaAdapterType == MFX_MEDIA_INTEGRATED)
+    {
+        return 1;
+    }
+    if (lhs->mediaAdapterType == MFX_MEDIA_INTEGRATED && rhs->mediaAdapterType != MFX_MEDIA_INTEGRATED)
+    {
+        return -1;
+    }
+
+    // prefer dll with lower API version
     if (lhs->actualApiVersion < rhs->actualApiVersion)
     {
         return -1;
@@ -179,7 +200,7 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
     HandleVector allocatedHandle;
     VectorHandleGuard handleGuard(allocatedHandle);
 
-    MFX_DISP_HANDLE *pHandle;
+    MFX_DISP_HANDLE_EX *pHandle;
     wchar_t dllName[MFX_MAX_DLL_PATH] = { 0 };
     MFX::MFXLibraryIterator libIterator;
 
@@ -191,6 +212,10 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
 
     // implementation interface masked from the input parameter
     mfxIMPL implInterface = par.Implementation & -MFX_IMPL_VIA_ANY;
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+    bool isSingleThread = (implInterface & MFX_IMPL_EXTERNAL_THREADING) > 0;
+    implInterface &= ~MFX_IMPL_EXTERNAL_THREADING;
+#endif
     mfxIMPL implInterfaceOrig = implInterface;
     mfxVersion requiredVersion = {{MFX_VERSION_MINOR, MFX_VERSION_MAJOR}};
 
@@ -218,7 +243,7 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
         *session = 0;
 
         // allocate the dispatching handle and call-table
-        pHandle = new MFX_DISP_HANDLE(requiredVersion);
+        pHandle = new MFX_DISP_HANDLE_EX(requiredVersion);
     }
     catch(...)
     {
@@ -234,6 +259,11 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
     maxImplIdx = implTypesRange[implMethod].maxIndex;
     do
     {
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+        if (isSingleThread && implTypes[curImplIdx].implType != MFX_LIB_HARDWARE)
+            continue;
+#endif
+
         int currentStorage = MFX::MFX_STORAGE_ID_FIRST;
         implInterface = implInterfaceOrig;
         do
@@ -280,19 +310,36 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
                     DISPATCHER_LOG_INFO((("loading library %S\n"), dllName));
                     // try to load the selected DLL
                     curImpl = implTypes[curImplIdx].impl;
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+                    if (isSingleThread)
+                        curImpl |= MFX_IMPL_EXTERNAL_THREADING;
+#endif
                     mfxRes = pHandle->LoadSelectedDLL(dllName, implType, curImpl, implInterface, par);
                     // unload the failed DLL
                     if (MFX_ERR_NONE != mfxRes)
                     {
                         pHandle->Close();
+                        continue;
                     }
-                    else
+
+                    mfxPlatform platform = { MFX_PLATFORM_UNKNOWN, 0, MFX_MEDIA_UNKNOWN };
+                    if (pHandle->callTable[eMFXVideoCORE_QueryPlatform])
                     {
-                        libIterator.GetSubKeyName(pHandle->subkeyName, sizeof(pHandle->subkeyName) / sizeof(pHandle->subkeyName[0]));
-                        pHandle->storageID = libIterator.GetStorageID();
-                        allocatedHandle.push_back(pHandle);
-                        pHandle = new MFX_DISP_HANDLE(requiredVersion);
+                        mfxRes = MFXVideoCORE_QueryPlatform((mfxSession)pHandle, &platform);
+                        if (MFX_ERR_NONE != mfxRes)
+                        {
+                            DISPATCHER_LOG_WRN(("MFXVideoCORE_QueryPlatform failed, rejecting loaded library\n"));
+                            pHandle->Close();
+                            continue;
+                        }
                     }
+                    pHandle->mediaAdapterType = platform.MediaAdapterType;
+                    DISPATCHER_LOG_INFO((("media adapter type is %d\n"), pHandle->mediaAdapterType));
+
+                    libIterator.GetSubKeyName(pHandle->subkeyName, sizeof(pHandle->subkeyName) / sizeof(pHandle->subkeyName[0]));
+                    pHandle->storageID = libIterator.GetStorageID();
+                    allocatedHandle.push_back(pHandle);
+                    pHandle = new MFX_DISP_HANDLE_EX(requiredVersion);
 
                 } while (MFX_ERR_NONE != mfxRes);
             }
@@ -302,7 +349,7 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
 
         } while ((MFX_ERR_NONE != mfxRes) && (MFX::MFX_STORAGE_ID_LAST >= currentStorage));
 
-    } while ((MFX_ERR_NONE != mfxRes) && (++curImplIdx <= maxImplIdx));
+    } while (++curImplIdx <= maxImplIdx);
 
     curImplIdx = implTypesRange[implMethod].minIndex;
     maxImplIdx = implTypesRange[implMethod].maxIndex;
@@ -310,6 +357,11 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
     // Load RT from app folder (libmfxsw64 with API >= 1.10)
     do
     {
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+        if (isSingleThread && implTypes[curImplIdx].implType != MFX_LIB_HARDWARE)
+            continue;
+#endif
+
         implInterface = implInterfaceOrig;
         // initialize the library iterator
         mfxRes = libIterator.Init(implTypes[curImplIdx].implType,
@@ -345,6 +397,10 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
 
                 // try to load the selected DLL
                 curImpl = implTypes[curImplIdx].impl;
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+                if (isSingleThread)
+                    curImpl |= MFX_IMPL_EXTERNAL_THREADING;
+#endif
                 mfxRes = pHandle->LoadSelectedDLL(dllName, implType, curImpl, implInterface, par);
                 // unload the failed DLL
                 if (MFX_ERR_NONE != mfxRes)
@@ -361,7 +417,7 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
                     }
                     pHandle->storageID = MFX::MFX_UNKNOWN_KEY;
                     allocatedHandle.push_back(pHandle);
-                    pHandle = new MFX_DISP_HANDLE(requiredVersion);
+                    pHandle = new MFX_DISP_HANDLE_EX(requiredVersion);
                 }
 
             } while (MFX_ERR_NONE != mfxRes);
@@ -373,6 +429,11 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
     curImplIdx = implTypesRange[implMethod].minIndex;
     do
     {
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+        if (isSingleThread && implTypes[curImplIdx].implType != MFX_LIB_HARDWARE)
+            continue;
+#endif
+
         implInterface = implInterfaceOrig;
 
         if (par.Implementation & MFX_IMPL_AUDIO)
@@ -406,10 +467,15 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
                 }
                 if (MFX_ERR_NONE == mfxRes)
                 {
+                    curImpl = implTypes[curImplIdx].impl;
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+                    if (isSingleThread)
+                        curImpl |= MFX_IMPL_EXTERNAL_THREADING;
+#endif
                     // try to load the selected DLL using default DLL search mechanism
                     mfxRes = pHandle->LoadSelectedDLL(dllName,
                         implTypes[curImplIdx].implType,
-                        implTypes[curImplIdx].impl,
+                        curImpl,
                         implInterface,
                         par);
                 }
@@ -423,7 +489,7 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
                 {
                     pHandle->storageID = MFX::MFX_UNKNOWN_KEY;
                     allocatedHandle.push_back(pHandle);
-                    pHandle = new MFX_DISP_HANDLE(requiredVersion);
+                    pHandle = new MFX_DISP_HANDLE_EX(requiredVersion);
                 }
         }
     }
@@ -442,9 +508,9 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
             if (HandleSort(&(*first), &(*it)) != 0)
                 NeedSort = true;
 
-        // select dll with version with lowest version number still greater or equal to requested
+        // sort allocatedHandle so that the most preferred dll is at the beginning
         if (NeedSort)
-            qsort(&(*allocatedHandle.begin()), allocatedHandle.size(), sizeof(MFX_DISP_HANDLE*), &HandleSort);
+            qsort(&(*allocatedHandle.begin()), allocatedHandle.size(), sizeof(MFX_DISP_HANDLE_EX*), &HandleSort);
     }
     HandleVector::iterator candidate = allocatedHandle.begin();
     // check the final result of loading
@@ -505,7 +571,41 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
 
     // everything is OK. Save pointers to the output variable
     *candidate = 0; // keep this one safe from guard destructor
-    *((MFX_DISP_HANDLE **) session) = pHandle;
+
+
+    //===================================
+
+    // MFXVideoCORE_QueryPlatform call creates d3d device handle, so we have handle right after MFXInit and can't accept external handle
+    // This is a workaround which calls close-init to remove that handle
+
+    mfxFunctionPointer *actualTable = (pHandle->impl & MFX_IMPL_AUDIO) ? pHandle->callAudioTable : pHandle->callTable;
+    mfxFunctionPointer pFunc;
+
+    pFunc = actualTable[eMFXClose];
+    mfxRes = (*(mfxStatus(MFX_CDECL *) (mfxSession)) pFunc) (pHandle->session);
+    if (mfxRes != MFX_ERR_NONE)
+        return mfxRes;
+
+    pHandle->session = 0;
+    bool callOldInit = (pHandle->impl & MFX_IMPL_AUDIO) || !actualTable[eMFXInitEx];
+    pFunc = actualTable[(callOldInit) ? eMFXInit : eMFXInitEx];
+
+    mfxVersion version(pHandle->apiVersion);
+    if (callOldInit)
+    {
+        pHandle->loadStatus = (*(mfxStatus(MFX_CDECL *) (mfxIMPL, mfxVersion *, mfxSession *)) pFunc) (pHandle->impl | pHandle->implInterface, &version, &pHandle->session);
+    }
+    else
+    {
+        mfxInitParam initPar = par;
+        initPar.Implementation = pHandle->impl | pHandle->implInterface;
+        initPar.Version = version;
+        pHandle->loadStatus = (*(mfxStatus(MFX_CDECL *) (mfxInitParam, mfxSession *)) pFunc) (initPar, &pHandle->session);
+    }
+
+    //===================================
+
+    *((MFX_DISP_HANDLE_EX **) session) = pHandle;
 
     return pHandle->loadStatus;
 
@@ -796,40 +896,42 @@ static mfxModuleHandle hModule;
 // InitialiseMediaSession() implemented in intel_gfx_api.dll
 mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
 {
-    HRESULT hr = S_OK;
-
 #if defined(MEDIASDK_ARM_LOADER)
-    hr = E_NOTIMPL;
+
+    return MFX_ERR_UNSUPPORTED;
+
 #else
+
     wchar_t IntelGFXAPIdllName[MFX_MAX_DLL_PATH] = { 0 };
+    mfxI32 adapterNum = -1;
 
-    DriverStoreLoader dsLoader;
-    if (!dsLoader.GetDriverStorePath(IntelGFXAPIdllName, sizeof(IntelGFXAPIdllName)))
+    switch (par.Implementation & 0xf)
     {
-        return MFX_ERR_UNSUPPORTED;
-    }
-
-    size_t pathLen = wcslen(IntelGFXAPIdllName);
-    mfx_get_default_intel_gfx_api_dll_name(IntelGFXAPIdllName + pathLen, sizeof(IntelGFXAPIdllName) / sizeof(IntelGFXAPIdllName[0]) - pathLen);
-    DISPATCHER_LOG_INFO((("loading %S\n"), IntelGFXAPIdllName));
-
-    hModule = MFX::mfx_dll_load(IntelGFXAPIdllName);
-    if (!hModule)
-    {
-        DISPATCHER_LOG_ERROR("Can't load intel_gfx_api\n");
-        return MFX_ERR_UNSUPPORTED;
-    }
-
-    mfxFunctionPointer pFunc = (mfxFunctionPointer)mfx_dll_get_addr(hModule, "InitialiseMediaSession");
-    if (!pFunc)
-    {
-        DISPATCHER_LOG_ERROR("Can't find required API function: InitialiseMediaSession\n");
-        return MFX_ERR_UNSUPPORTED;
-    }
-    hr = (*(HRESULT(APIENTRY *) (HANDLE*, LPVOID, LPVOID)) pFunc) ((HANDLE*)session, &par, NULL);
+    case MFX_IMPL_SOFTWARE:
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+    case MFX_IMPL_SINGLE_THREAD:
 #endif
+        return MFX_ERR_UNSUPPORTED;
+    case MFX_IMPL_AUTO:
+    case MFX_IMPL_HARDWARE:
+        adapterNum = 0;
+        break;
+    case MFX_IMPL_HARDWARE2:
+        adapterNum = 1;
+        break;
+    case MFX_IMPL_HARDWARE3:
+        adapterNum = 2;
+        break;
+    case MFX_IMPL_HARDWARE4:
+        adapterNum = 3;
+        break;
+    default:
+        return GfxApiInitPriorityIntegrated(par, session, hModule);
+    }
 
-    return (hr == S_OK) ? MFX_ERR_NONE : (mfxStatus)hr;
+    return GfxApiInitByAdapterNum(par, adapterNum, session, hModule);
+
+#endif
 }
 
 // for the UWP_DISPATCHER purposes implementation of MFXClose is calling
@@ -840,27 +942,20 @@ mfxStatus MFXClose(mfxSession session)
         return MFX_ERR_INVALID_HANDLE;
     }
 
-    HRESULT hr = S_OK;
+    mfxStatus sts = MFX_ERR_NONE;
 
 #if defined(MEDIASDK_ARM_LOADER)
-    hr = E_NOTIMPL;
+
+    sts = MFX_ERR_UNSUPPORTED;
+
 #else
-    if (hModule)
-    {
-        mfxFunctionPointer pFunc = (mfxFunctionPointer)mfx_dll_get_addr(hModule, "DisposeMediaSession");
-        if (!pFunc)
-        {
-            DISPATCHER_LOG_ERROR("Can't find required API function: DisposeMediaSession\n");
-            return MFX_ERR_INVALID_HANDLE;
-        }
-        hr = (*(HRESULT(APIENTRY *) (HANDLE)) pFunc) ((HANDLE)session);
-    }
-    else
-        return MFX_ERR_INVALID_HANDLE;
+
+    sts = GfxApiClose(session, hModule);
+
 #endif
 
     session = (mfxSession)NULL;
-    return (hr == S_OK) ? MFX_ERR_NONE : mfxStatus(hr);
+    return sts;
 }
 
 #undef FUNCTION
@@ -899,7 +994,7 @@ mfxStatus MFXJoinSession(mfxSession session, mfxSession child_session)
     MFX_DISP_HANDLE *pChildHandle = (MFX_DISP_HANDLE *)child_session;
 
     // get the function's address and make a call
-    if ((pHandle) && (pChildHandle) && (pHandle->apiVersion == pChildHandle->apiVersion))
+    if ((pHandle) && (pChildHandle) && (pHandle->actualApiVersion == pChildHandle->actualApiVersion))
     {
         /* check whether it is audio session or video */
         int tableIndex = eMFXJoinSession;
