@@ -26,14 +26,14 @@ or https://software.intel.com/en-us/media-client-solutions-support.
 #include "vaapi_utils_android.h"
 #endif
 
-CHWDevice* CreateVAAPIDevice(int type = MFX_LIBVA_DRM);
+CHWDevice* CreateVAAPIDevice(const std::string& devicePath = "", int type = MFX_LIBVA_DRM);
 
 #if defined(LIBVA_DRM_SUPPORT)
 /** VAAPI DRM implementation. */
 class CVAAPIDeviceDRM : public CHWDevice
 {
 public:
-    CVAAPIDeviceDRM(int type);
+    CVAAPIDeviceDRM(const std::string& devicePath, int type);
     virtual ~CVAAPIDeviceDRM(void);
 
     virtual mfxStatus Init(mfxHDL hWindow, mfxU16 nViews, mfxU32 nAdapterNum);
@@ -130,8 +130,6 @@ private:
 
 class Wayland;
 
-#define HANDLE_WAYLAND_DRIVER   (MFX_HANDLE_VA_DISPLAY << 4)
-
 class CVAAPIDeviceWayland : public CHWDevice
 {
 public:
@@ -155,11 +153,9 @@ public:
         if((MFX_HANDLE_VA_DISPLAY == type) && (NULL != pHdl)) {
             *pHdl = m_DRMLibVA.GetVADisplay();
             return MFX_ERR_NONE;
-        } else if((HANDLE_WAYLAND_DRIVER  == type) && (NULL != m_Wayland)) {
-            *pHdl = m_Wayland;
-            return MFX_ERR_NONE;
-    }
-    return MFX_ERR_UNSUPPORTED;
+        }
+
+        return MFX_ERR_UNSUPPORTED;
     }
     virtual mfxStatus RenderFrame(mfxFrameSurface1 * pSurface, mfxFrameAllocator * pmfxAlloc);
     virtual void UpdateTitle(double fps) { }
@@ -169,6 +165,10 @@ public:
         m_isMondelloInputEnabled = isMondelloInputEnabled;
     }
 
+    Wayland * GetWaylandHandle()
+    {
+        return m_Wayland;
+    }
 protected:
     DRMLibVA m_DRMLibVA;
     MfxLoader::VA_WaylandClientProxy  m_WaylandClient;
