@@ -1,5 +1,5 @@
 /******************************************************************************\
-Copyright (c) 2005-2019, Intel Corporation
+Copyright (c) 2005-2020, Intel Corporation
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -114,6 +114,10 @@ mfxStatus CSmplYUVReader::Init(std::list<msdk_string> inputs, mfxU32 ColorFormat
 #if (MFX_VERSION >= 1027)
         && MFX_FOURCC_Y210 != ColorFormat
         && MFX_FOURCC_Y410 != ColorFormat
+#endif
+#if (MFX_VERSION >= 1031)
+        && MFX_FOURCC_P016 != ColorFormat
+        && MFX_FOURCC_Y216 != ColorFormat
 #endif
         )
     {
@@ -124,6 +128,10 @@ mfxStatus CSmplYUVReader::Init(std::list<msdk_string> inputs, mfxU32 ColorFormat
         || MFX_FOURCC_P210 == ColorFormat
 #if (MFX_VERSION >= 1027)
         || MFX_FOURCC_Y210 == ColorFormat
+#endif
+#if (MFX_VERSION >= 1031)
+        || MFX_FOURCC_P016 == ColorFormat
+        || MFX_FOURCC_Y216 == ColorFormat
 #endif
     )
     {
@@ -203,6 +211,9 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
     mfxFrameInfo& pInfo = pSurface->Info;
     mfxFrameData& pData = pSurface->Data;
 
+    mfxU32 shiftSizeLuma   = 16 - pInfo.BitDepthLuma;
+    mfxU32 shiftSizeChroma = 16 - pInfo.BitDepthChroma;
+
     mfxU32 vid = pInfo.FrameId.ViewId;
 
     if (vid > m_files.size())
@@ -221,7 +232,11 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
         h = pInfo.Height;
     }
 
-    mfxU32 nBytesPerPixel = (pInfo.FourCC == MFX_FOURCC_P010 || pInfo.FourCC == MFX_FOURCC_P210 ) ? 2 : 1;
+    mfxU32 nBytesPerPixel = (pInfo.FourCC == MFX_FOURCC_P010 || pInfo.FourCC == MFX_FOURCC_P210
+#if (MFX_VERSION >= 1031)
+        || pInfo.FourCC == MFX_FOURCC_P016
+#endif
+    ) ? 2 : 1;
 
     if (   MFX_FOURCC_YUY2 == pInfo.FourCC
         || MFX_FOURCC_UYVY == pInfo.FourCC
@@ -232,6 +247,9 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
 #if (MFX_VERSION >= 1027)
         || MFX_FOURCC_Y210 == pInfo.FourCC
         || MFX_FOURCC_Y410 == pInfo.FourCC
+#endif
+#if (MFX_VERSION >= 1031)
+        || MFX_FOURCC_Y216 == pInfo.FourCC
 #endif
     )
     {
@@ -290,8 +308,15 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
 #if (MFX_VERSION >= 1027)
         case MFX_FOURCC_Y210:
         case MFX_FOURCC_Y410:
+#if (MFX_VERSION >= 1031)
+        case MFX_FOURCC_Y216:
+#endif
             pitch = pData.Pitch;
-            ptr = ((pInfo.FourCC== MFX_FOURCC_Y210)  ? pData.Y : (mfxU8*)pData.Y410) + pInfo.CropX*4 + pInfo.CropY * pData.Pitch;
+            ptr = ((pInfo.FourCC == MFX_FOURCC_Y210
+#if (MFX_VERSION >= 1031)
+            || pInfo.FourCC == MFX_FOURCC_Y216
+#endif
+            )  ? pData.Y : (mfxU8*)pData.Y410) + pInfo.CropX*4 + pInfo.CropY * pData.Pitch;
 
             for (i = 0; i < h; i++)
             {
@@ -302,12 +327,16 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
                     return MFX_ERR_MORE_DATA;
                 }
 
-                if (MFX_FOURCC_Y210 == pInfo.FourCC && shouldShift10BitsHigh)
+                if ((MFX_FOURCC_Y210 == pInfo.FourCC
+#if (MFX_VERSION >= 1031)
+                    || MFX_FOURCC_Y216 == pInfo.FourCC
+#endif
+                ) && shouldShift10BitsHigh)
                 {
                     mfxU16* shortPtr = (mfxU16*)(ptr + i * pitch);
                     for (int idx = 0; idx < w*2; idx++)
                     {
-                        shortPtr[idx] <<= 6;
+                        shortPtr[idx] <<= shiftSizeLuma;
                     }
                 }
 
@@ -318,7 +347,14 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
             return MFX_ERR_UNSUPPORTED;
         }
     }
-    else if (MFX_FOURCC_NV12 == pInfo.FourCC || MFX_FOURCC_YV12 == pInfo.FourCC || MFX_FOURCC_P010 == pInfo.FourCC || MFX_FOURCC_P210 == pInfo.FourCC)
+    else if (MFX_FOURCC_NV12 == pInfo.FourCC
+            || MFX_FOURCC_YV12 == pInfo.FourCC
+            || MFX_FOURCC_P010 == pInfo.FourCC
+            || MFX_FOURCC_P210 == pInfo.FourCC
+#if (MFX_VERSION >= 1031)
+            || MFX_FOURCC_P016 == pInfo.FourCC
+#endif
+    )
     {
         pitch = pData.Pitch;
         ptr = pData.Y + pInfo.CropX + pInfo.CropY * pData.Pitch;
@@ -334,12 +370,17 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
             }
 
             // Shifting data if required
-            if((MFX_FOURCC_P010 == pInfo.FourCC || MFX_FOURCC_P210 == pInfo.FourCC) && shouldShift10BitsHigh)
+            if((MFX_FOURCC_P010 == pInfo.FourCC
+             || MFX_FOURCC_P210 == pInfo.FourCC
+#if (MFX_VERSION >= 1031)
+             || MFX_FOURCC_P016 == pInfo.FourCC
+#endif
+            ) && shouldShift10BitsHigh)
             {
                 mfxU16* shortPtr = (mfxU16*)(ptr + i * pitch);
                 for(int idx = 0; idx < w; idx++)
                 {
-                    shortPtr[idx]<<=6;
+                    shortPtr[idx]<<=shiftSizeLuma;
                 }
             }
         }
@@ -442,6 +483,9 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
         case MFX_FOURCC_NV12:
         case MFX_FOURCC_P010:
         case MFX_FOURCC_P210:
+#if (MFX_VERSION >= 1031)
+        case MFX_FOURCC_P016:
+#endif
             if (MFX_FOURCC_P210 != pInfo.FourCC)
             {
                 h /= 2;
@@ -457,12 +501,17 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface)
                 }
 
                 // Shifting data if required
-                if((MFX_FOURCC_P010 == pInfo.FourCC || MFX_FOURCC_P210 == pInfo.FourCC) && shouldShift10BitsHigh)
+            if((MFX_FOURCC_P010 == pInfo.FourCC
+             || MFX_FOURCC_P210 == pInfo.FourCC
+#if (MFX_VERSION >= 1031)
+             || MFX_FOURCC_P016 == pInfo.FourCC
+#endif
+             ) && shouldShift10BitsHigh)
                 {
                     mfxU16* shortPtr = (mfxU16*)(ptr + i * pitch);
                     for(int idx = 0; idx < w; idx++)
                     {
-                        shortPtr[idx]<<=6;
+                        shortPtr[idx]<<=shiftSizeChroma;
                     }
                 }
             }
@@ -739,11 +788,8 @@ CIVFFrameReader::CIVFFrameReader()
         return MFX_ERR_MORE_DATA;\
 }\
 
-mfxStatus CIVFFrameReader::Init(const msdk_char *strFileName)
+mfxStatus CIVFFrameReader::ReadHeader()
 {
-    mfxStatus sts = CSmplBitstreamReader::Init(strFileName);
-    MSDK_CHECK_STATUS(sts, "CSmplBitstreamReader::Init failed");
-
     // read and skip IVF header
     READ_BYTES(&m_hdr.dkif, sizeof(m_hdr.dkif));
     READ_BYTES(&m_hdr.version, sizeof(m_hdr.version));
@@ -756,11 +802,28 @@ mfxStatus CIVFFrameReader::Init(const msdk_char *strFileName)
     READ_BYTES(&m_hdr.num_frames, sizeof(m_hdr.num_frames));
     READ_BYTES(&m_hdr.unused, sizeof(m_hdr.unused));
     MSDK_CHECK_NOT_EQUAL(fseek(m_fSource, m_hdr.header_len, SEEK_SET), 0, MFX_ERR_UNSUPPORTED);
+    return MFX_ERR_NONE;
+}
+
+void CIVFFrameReader::Reset()
+{
+    CSmplBitstreamReader::Reset();
+    std::ignore = ReadHeader();
+}
+
+mfxStatus CIVFFrameReader::Init(const msdk_char *strFileName)
+{
+    mfxStatus sts = CSmplBitstreamReader::Init(strFileName);
+    MSDK_CHECK_STATUS(sts, "CSmplBitstreamReader::Init failed");
+
+    sts = ReadHeader();
+    MSDK_CHECK_STATUS(sts, "CIVFFrameReader::ReadHeader failed");
 
     // check header
     MSDK_CHECK_NOT_EQUAL(MFX_MAKEFOURCC('D','K','I','F'), m_hdr.dkif, MFX_ERR_UNSUPPORTED);
     if ((m_hdr.codec_FourCC != MFX_MAKEFOURCC('V','P','8','0')) &&
-        (m_hdr.codec_FourCC != MFX_MAKEFOURCC('V','P','9','0')))
+        (m_hdr.codec_FourCC != MFX_MAKEFOURCC('V','P','9','0')) &&
+        (m_hdr.codec_FourCC != MFX_MAKEFOURCC('A','V','0','1')))
     {
         return MFX_ERR_UNSUPPORTED;
     }
@@ -914,17 +977,26 @@ mfxStatus GetChromaSize(const mfxFrameInfo & pInfo, mfxU32 & ChromaW, mfxU32 & C
         break;
     }
     case MFX_FOURCC_P010:
-    case MFX_FOURCC_P210:
     case MFX_FOURCC_P016:
     {
         ChromaW = (pInfo.CropW % 2) ? (pInfo.CropW + 1) : pInfo.CropW;
-        ChromaH = pInfo.FourCC == MFX_FOURCC_P210 ? (mfxU32)pInfo.CropH : (mfxU32)(pInfo.CropH + 1) / 2;
+        ChromaH = (mfxU32)(pInfo.CropH + 1) / 2;
+        break;
+    }
+
+    case MFX_FOURCC_P210:
+    case MFX_FOURCC_Y210:
+    case MFX_FOURCC_Y216:
+    {
+        ChromaW = (pInfo.CropW % 2) ? (pInfo.CropW + 1) : pInfo.CropW;
+        ChromaH = pInfo.CropH;
         break;
     }
 
     case MFX_FOURCC_RGB4:
     case MFX_FOURCC_AYUV:
     case MFX_FOURCC_YUY2:
+    case MFX_FOURCC_NV16:
     case MFX_FOURCC_A2RGB10:
     case MFX_FOURCC_Y410:
     case MFX_FOURCC_Y416:
@@ -985,6 +1057,7 @@ mfxStatus CSmplYUVWriter::WriteNextFrame(mfxFrameSurface1 *pSurface)
     {
     case MFX_FOURCC_YV12:
     case MFX_FOURCC_NV12:
+    case MFX_FOURCC_NV16:
         for (i = 0; i < pInfo.CropH; i++)
         {
             MSDK_CHECK_NOT_EQUAL(
@@ -1143,6 +1216,16 @@ mfxStatus CSmplYUVWriter::WriteNextFrame(mfxFrameSurface1 *pSurface)
         }
         break;
     }
+    case MFX_FOURCC_NV16:
+    {
+        for (i = 0; i < ChromaH; i++)
+        {
+            MSDK_CHECK_NOT_EQUAL(
+                fwrite(pData.UV + (pInfo.CropY * pData.Pitch / 2 + pInfo.CropX) + i * pData.Pitch, 1, ChromaW, dstFile),
+                ChromaW, MFX_ERR_UNDEFINED_BEHAVIOR);
+        }
+        break;
+    }
     case MFX_FOURCC_P010:
 #if (MFX_VERSION >= 1031)
     case MFX_FOURCC_P016:
@@ -1743,6 +1826,12 @@ const msdk_char* ColorFormatToStr(mfxU32 format)
         return MSDK_STRING("Y210");
     case MFX_FOURCC_Y410:
         return MSDK_STRING("Y410");
+#endif
+#if (MFX_VERSION >= 1031)
+    case MFX_FOURCC_P016:
+        return MSDK_STRING("P016");
+    case MFX_FOURCC_Y216:
+        return MSDK_STRING("Y216");
 #endif
     default:
         return MSDK_STRING("unsupported");
@@ -1922,45 +2011,47 @@ mfxVersion getMinimalRequiredVersion(const APIChangeFeatures &features)
 
 bool CheckVersion(mfxVersion* version, msdkAPIFeature feature)
 {
-    if (!version->Major || (version->Major > 1)) {
+    if (!version) {
         return false;
     }
 
+    mfxU32 ver = MakeVersion(version->Major, version->Minor);
+
     switch (feature) {
     case MSDK_FEATURE_NONE:
         return true;
     case MSDK_FEATURE_MVC:
-        if ((version->Major == 1) && (version->Minor >= 3)) {
+        if (ver >= 1003) {
             return true;
         }
         break;
     case MSDK_FEATURE_JPEG_DECODE:
-        if ((version->Major == 1) && (version->Minor >= 3)) {
+        if (ver >= 1003) {
             return true;
         }
         break;
    case MSDK_FEATURE_LOW_LATENCY:
-        if ((version->Major == 1) && (version->Minor >= 3)) {
+        if (ver >= 1003) {
             return true;
         }
         break;
     case MSDK_FEATURE_MVC_VIEWOUTPUT:
-        if ((version->Major == 1) && (version->Minor >= 4)) {
+        if (ver >= 1004) {
             return true;
         }
         break;
     case MSDK_FEATURE_JPEG_ENCODE:
-        if ((version->Major == 1) && (version->Minor >= 6)) {
+        if (ver >= 1006) {
             return true;
         }
         break;
     case MSDK_FEATURE_LOOK_AHEAD:
-        if ((version->Major == 1) && (version->Minor >= 7)) {
+        if (ver >= 1007) {
             return true;
         }
         break;
     case MSDK_FEATURE_PLUGIN_API:
-        if ((version->Major == 1) && (version->Minor >= 8)) {
+        if (ver >= 1008) {
             return true;
         }
         break;
@@ -2191,6 +2282,7 @@ bool IsDecodeCodecSupported(mfxU32 codecFormat)
         case MFX_CODEC_JPEG:
         case MFX_CODEC_VP8:
         case MFX_CODEC_VP9:
+        case MFX_CODEC_AV1:
         break;
     default:
         return false;
@@ -2275,6 +2367,10 @@ mfxStatus StrFormatToCodecFormatFourCC(msdk_char* strInput, mfxU32 &codecFormat)
         {
             codecFormat = MFX_CODEC_VP9;
         }
+        else if (0 == msdk_strcmp(strInput, MSDK_STRING("av1")))
+        {
+            codecFormat = MFX_CODEC_AV1;
+        }
         else if ((0 == msdk_strcmp(strInput, MSDK_STRING("raw"))))
         {
             codecFormat = MFX_CODEC_DUMP;
@@ -2283,6 +2379,14 @@ mfxStatus StrFormatToCodecFormatFourCC(msdk_char* strInput, mfxU32 &codecFormat)
         {
             codecFormat = MFX_CODEC_RGB4;
         }
+        else if ((0 == msdk_strcmp(strInput, MSDK_STRING("nv12"))))
+        {
+            codecFormat = MFX_CODEC_NV12;
+        }
+        else if ((0 == msdk_strcmp(strInput, MSDK_STRING("i420"))))
+        {
+            codecFormat = MFX_CODEC_I420;
+        }
         else
             sts = MFX_ERR_UNSUPPORTED;
     }
@@ -2588,11 +2692,17 @@ mfxU16 FourCCToChroma(mfxU32 fourCC)
     {
     case MFX_FOURCC_NV12:
     case MFX_FOURCC_P010:
+#if (MFX_VERSION >= 1031)
+    case MFX_FOURCC_P016:
+#endif
         return MFX_CHROMAFORMAT_YUV420;
     case MFX_FOURCC_NV16:
     case MFX_FOURCC_P210:
 #if (MFX_VERSION >= 1027)
     case MFX_FOURCC_Y210:
+#endif
+#if (MFX_VERSION >= 1031)
+    case MFX_FOURCC_Y216:
 #endif
     case MFX_FOURCC_YUY2:
     case MFX_FOURCC_UYVY:
