@@ -288,7 +288,7 @@ mfxStatus va_to_mfx_status(VAStatus va_res)
 }
 
 #if defined(LIBVA_DRM_SUPPORT) || defined(LIBVA_X11_SUPPORT)
-CLibVA* CreateLibVA(int type)
+CLibVA* CreateLibVA(const std::string& devicePath, int type)
 {
     CLibVA * libva = NULL;
     switch (type)
@@ -297,7 +297,7 @@ CLibVA* CreateLibVA(int type)
 #if defined(LIBVA_DRM_SUPPORT)
         try
         {
-            libva = new DRMLibVA(type);
+            libva = new DRMLibVA(devicePath, type);
         }
         catch (std::exception&)
         {
@@ -335,7 +335,7 @@ CLibVA* CreateLibVA(int type)
         {
             try
             {
-                libva = new DRMLibVA(type);
+                libva = new DRMLibVA(devicePath, type);
             }
             catch (std::exception&)
             {
