@@ -251,6 +251,8 @@ namespace TranscodingSample
         mfxU16 MaxKbps;
         mfxU16 InitialDelayInKB;
         mfxU16 GopOptFlag;
+        mfxU16 AdaptiveI;
+        mfxU16 AdaptiveB;
 
         mfxU16 WeightedPred;
         mfxU16 WeightedBiPred;
@@ -283,6 +285,10 @@ namespace TranscodingSample
         mfxU16 nQPP;
         mfxU16 nQPB;
         bool bDisableQPOffset;
+        mfxU16 QVBRQuality;
+        mfxU16 ICQQuality;
+        mfxU16 Convergence;
+        mfxU16 Accuracy;
 
         mfxU16 nAvcTemp;
         mfxU16 nBaseLayerPID;
@@ -311,6 +317,7 @@ namespace TranscodingSample
 
         bool bUseOpaqueMemory;
         bool bForceSysMem;
+        mfxU16 DecOutPattern;
         mfxU16 VppOutPattern;
         mfxU16 nGpuCopyMode;
 
@@ -355,6 +362,8 @@ namespace TranscodingSample
 
         EPresetModes PresetMode;
         bool shouldPrintPresets;
+
+        bool rawInput;
     };
 
     struct sInputParams: public __sInputParams
@@ -720,6 +729,7 @@ namespace TranscodingSample
         mfxStatus InitPluginMfxParams(sInputParams *pInParams);
         mfxStatus InitPreEncMfxParams(sInputParams *pInParams);
 
+        virtual mfxU32 FileFourCC2EncFourCC(mfxU32 fcc);
         void FillFrameInfoForEncoding(mfxFrameInfo& info, sInputParams *pInParams);
 
         mfxStatus AllocAndInitVppDoNotUse(sInputParams *pInParams);
@@ -862,6 +872,8 @@ namespace TranscodingSample
 
         bool m_bInsertIDR;
 
+        bool m_rawInput;
+
         std::unique_ptr<ExtendedBSStore>        m_pBSStore;
 
         mfxU32                                m_FrameNumberPreference;
