@@ -1,5 +1,5 @@
 /******************************************************************************\
-Copyright (c) 2005-2019, Intel Corporation
+Copyright (c) 2005-2020, Intel Corporation
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -141,9 +141,16 @@ mfxStatus Launcher::Init(int argc, msdk_char *argv[])
     // check available adapters
     sts = QueryAdapters();
     MSDK_CHECK_STATUS(sts, "QueryAdapters failed");
+
+    if (m_eDevType && m_DisplaysData.empty())
+    {
+        msdk_printf(MSDK_STRING("No adapters found. HW-accelerated transcoding is impossible.\n"));
+        return MFX_ERR_UNSUPPORTED;
+    }
 #endif
 
 #if defined(_WIN32) || defined(_WIN64)
+    ForceImplForSession(0);
     if (m_eDevType == MFX_HANDLE_D3D9_DEVICE_MANAGER)
     {
         m_pAllocParam.reset(new D3DAllocatorParams);
@@ -153,12 +160,14 @@ mfxStatus Launcher::Init(int argc, msdk_char *argv[])
         if (m_InputParamsArray[m_InputParamsArray.size() -1].eModeExt == VppCompOnly)
         {
             /* Rendering case */
-            sts = m_hwdev->Init(NULL, 1, MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D9) );
+            sts = m_hwdev->Init(NULL, 1,
+                MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D9 | MFX_IMPL_BASETYPE(m_InputParamsArray[0].libType)) );
             m_InputParamsArray[m_InputParamsArray.size() -1].m_hwdev = m_hwdev.get();
         }
         else /* NO RENDERING*/
         {
-            sts = m_hwdev->Init(NULL, 0, MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D9) );
+            sts = m_hwdev->Init(NULL, 0,
+                MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D9 | MFX_IMPL_BASETYPE(m_InputParamsArray[0].libType)) );
         }
         MSDK_CHECK_STATUS(sts, "m_hwdev->Init failed");
         sts = m_hwdev->GetHandle(MFX_HANDLE_D3D9_DEVICE_MANAGER, (mfxHDL*)&hdl);
@@ -178,12 +187,14 @@ mfxStatus Launcher::Init(int argc, msdk_char *argv[])
         if (m_InputParamsArray[m_InputParamsArray.size() -1].eModeExt == VppCompOnly)
         {
             /* Rendering case */
-            sts = m_hwdev->Init(NULL, 1, MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D11) );
+            sts = m_hwdev->Init(NULL, 1,
+                MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D11 | MFX_IMPL_BASETYPE(m_InputParamsArray[0].libType)) );
             m_InputParamsArray[m_InputParamsArray.size() -1].m_hwdev = m_hwdev.get();
         }
         else /* NO RENDERING*/
         {
-            sts = m_hwdev->Init(NULL, 0, MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D11) );
+            sts = m_hwdev->Init(NULL, 0,
+                MSDKAdapter::GetNumber(0,MFX_IMPL_VIA_D3D11 | MFX_IMPL_BASETYPE(m_InputParamsArray[0].libType)) );
         }
         MSDK_CHECK_STATUS(sts, "m_hwdev->Init failed");
         sts = m_hwdev->GetHandle(MFX_HANDLE_D3D11_DEVICE, (mfxHDL*)&hdl);
@@ -327,13 +338,15 @@ mfxStatus Launcher::Init(int argc, msdk_char *argv[])
 
         std::unique_ptr<CSmplBitstreamReader> reader;
         std::unique_ptr<CSmplYUVReader> yuvreader;
-        if (m_InputParamsArray[i].DecodeId == MFX_CODEC_VP9)
+        if (m_InputParamsArray[i].DecodeId == MFX_CODEC_VP9 || m_InputParamsArray[i].DecodeId == MFX_CODEC_AV1)
         {
             reader.reset(new CIVFFrameReader());
         }
-        else if (m_InputParamsArray[i].DecodeId == MFX_CODEC_RGB4)
+        else if (m_InputParamsArray[i].DecodeId == MFX_CODEC_RGB4 ||
+                 m_InputParamsArray[i].DecodeId == MFX_CODEC_I420 ||
+                 m_InputParamsArray[i].DecodeId == MFX_CODEC_NV12)
         {
-            // YUV reader for RGB4 overlay
+            // YUV reader for RGB4 overlay and raw input
             yuvreader.reset(new CSmplYUVReader());
         }
         else
@@ -352,7 +365,7 @@ mfxStatus Launcher::Init(int argc, msdk_char *argv[])
         {
             std::list<msdk_string> input;
             input.push_back(m_InputParamsArray[i].strSrcFile);
-            sts = yuvreader->Init(input, MFX_FOURCC_RGB4);
+            sts = yuvreader->Init(input, m_InputParamsArray[i].DecodeId);
             MSDK_CHECK_STATUS(sts, "m_YUVReader->Init failed");
             sts = m_pExtBSProcArray.back()->SetReader(yuvreader);
             MSDK_CHECK_STATUS(sts, "m_pExtBSProcArray.back()->SetReader failed");
@@ -710,6 +723,12 @@ mfxStatus Launcher::QueryAdapters()
     mfxStatus sts = MFXQueryAdaptersNumber(&num_adapters_available);
     MFX_CHECK_STS(sts);
 
+    // no adapters on the machine, able to use software implementation
+    if (!num_adapters_available)
+    {
+        return MFX_ERR_NONE;
+    }
+
     m_DisplaysData.resize(num_adapters_available);
     m_Adapters = { m_DisplaysData.data(), mfxU32(m_DisplaysData.size()), 0u };
 
@@ -721,6 +740,9 @@ mfxStatus Launcher::QueryAdapters()
 
 void Launcher::ForceImplForSession(mfxU32 idxSession)
 {
+    if (m_InputParamsArray[idxSession].libType == MFX_IMPL_SOFTWARE)
+        return;
+
     //change only 8 bit of the implementation. Don't touch type of frames
     mfxIMPL impl = m_InputParamsArray[idxSession].libType & mfxI32(~0xFF);
 
@@ -1109,7 +1131,12 @@ mfxStatus Launcher::CreateSafetyBuffers()
 
 void Launcher::Close()
 {
-    m_pThreadContextArray.clear();
+    while(m_pThreadContextArray.size())
+    {
+        m_pThreadContextArray[m_pThreadContextArray.size() - 1].reset();
+        m_pThreadContextArray.pop_back();
+    }
+
     m_pAllocArray.clear();
     m_pBufferArray.clear();
     m_pExtBSProcArray.clear();
